
#include "Utility.h"
#include <cfloat>

static std::mt19937 randgen;

float shiftAngle(float angle)
{
	while(angle < -PI || angle > PI) {
		if(angle < -PI) { 
			angle += TAU;
		} else if(angle > PI) {
			angle -= TAU;
		}
	}

	return angle;
}

void initRandom()
{
	randgen.seed((unsigned long)time(NULL));
}

float getRandom()
{
	return (float)randgen()/(float)randgen.max();
}

float getRandom(float begin, float end)
{
	return getRandom()*(end-begin)+begin;
}

bool IsValidProperty(const std::string& prop)
{
	return !prop.empty() && prop.compare("No such property!") != 0;
}

int getQuadrant(const sf::Vector2f &p1, const sf::Vector2f &p2) {
	if(p2.x > p1.x && p2.y > p1.y) {
		return 1;
	} else if(p2.x < p1.x && p2.y > p1.y) {
		return 2;
	} else if(p2.x < p1.x && p2.y < p1.y) {
		return 3;
	} else if(p2.x > p1.x && p2.y < p1.y) {
		return 4;
	} else {
		// ???
		return -1;
	}
}

sf::Vector2f getQuadrantVec(int quadrant, float magnitude) {
	switch(quadrant) {
		case 1:
			return sf::Vector2f(magnitude,magnitude);
			break;
		case 2:
			return sf::Vector2f(-magnitude,magnitude);
			break;
		case 3:
			return sf::Vector2f(-magnitude,-magnitude);
			break;
		case 4:
			return sf::Vector2f(magnitude,-magnitude);
			break;
		default:
			return sf::Vector2f(0,0);
			break;
	}
	return sf::Vector2f(0,0);
}

float lineDistance(sf::Vector2f point, sf::Vector2f p1, sf::Vector2f p2) {
	sf::Vector2f v1 = point - p1;
	sf::Vector2f v2 = p2 - p1;
	float lensq = v2.x * v2.x + v2.y * v2.y;
	if(lensq == 0.0f) return magnitude(v1);

	//Get normalized scalar projection
	float t = dot(v1, v2) / lensq;

	if(t < 0.0f) return magnitude(point - p1);
	if(t > 1.0f) return magnitude(point - p2);

	sf::Vector2f projection = p1 + t * (p2 - p1);
	return magnitude(point - projection);
}

float lineDistance(sf::Vector2f p1, sf::Vector2f p2, sf::Vector2f p3, sf::Vector2f p4) {
#if DEBUG
	sf::Vector2f intersect = lineIntersect(p1, p2, p3, p4);
	assert(intersect.x != FLT_MAX && intersect.y != FLT_MAX);
#endif

	return std::min(lineDistance(p1, p3, p4),
			std::min(lineDistance(p2, p3, p4),
				std::min(lineDistance(p3, p1, p2),
					lineDistance(p4, p1, p2))));
}

bool onLine(sf::Vector2f p1, sf::Vector2f p2, sf::Vector2f point, float epsilon) {
	float crossproduct = (point.y - p1.y) * (p2.x - p1.x) - (point.x - p1.x) * (p2.y - p1.y);

	return std::abs(crossproduct) < epsilon &&
		point.x >= std::min(p1.x, p2.x) - epsilon &&
		point.x <= std::max(p1.x, p2.x) + epsilon &&
		point.y >= std::min(p1.y, p2.y) - epsilon &&
		point.y <= std::max(p1.y, p2.y) + epsilon;
}

bool contains(const std::vector<sf::Vector2f> &poly, sf::Vector2f point) {
	int i, j;
	bool c = false;

	for (i = 0, j = poly.size()-1; i < poly.size(); j = i++) {
		if ( ((poly[i].y>point.y) != (poly[j].y>point.y)) && (point.x < (poly[j].x-poly[i].x) * (point.y-poly[i].y) / (poly[j].y-poly[i].y) + poly[i].x) )
			c = !c;
	}

	return c;
}

//C is the vector in the middle
float angleBetween(sf::Vector2f base, sf::Vector2f vec, sf::Vector2f mid) {
	vec = vec - mid;
	base = base - mid;
	return angleBetween(base, vec);
}

float angleBetween(sf::Vector2f base, sf::Vector2f vec) {
	sf::Vector2f perp(-base.y, base.x);
	float proj_base = dot(vec, base);
	float proj_perp = dot(vec, perp);
	return atan2f(proj_perp, proj_base);
}
