
#include "Utility/Utility.h"
#include "Bullet.h"
#include "Level.h"
#include "Entity.h"
#include "Unit.h"

Bullet::Bullet(sf::Vector2f pos, sf::Time lifetime,
	float angle, float moveSpeed, float tracerLength, float power) :
		angle(angle),
		moveSpeed(moveSpeed),
		tracerLength(tracerLength),
		power(power),
		origin(pos),
		tracer(sf::Lines, 2),
		lifetime(lifetime) {
	tracer[0] = sf::Vertex(pos, sf::Color(255, 128, 128));
	tracer[1] = sf::Vertex(pos, sf::Color::Red);
	active = true;
	intersected = false;
	hasDamaged = false;
}

bool Bullet::IsActive() {
	return active;
}

void Bullet::Update(Level &level, const sf::Time &timePassed) {
	timer += timePassed;
	if(timer >= lifetime) {
		active = false;
	}

	sf::Vertex &head = tracer[0];
	sf::Vertex &tail = tracer[1];
	float moveMag = moveSpeed * timePassed.asSeconds();
	sf::Vector2f movement = sf::Vector2f(moveMag * cos(angle*TO_RAD), moveMag * sin(angle*TO_RAD));
	sf::Vector2f intersect;

	if(!intersected) {
		bool found = level.GetCollideAngle(head.position, angle*TO_RAD, intersect, COLLIDE_WALLS|COLLIDE_ENEMIES);
		sf::Vector2f dist = intersect - head.position;
		if(found && magnitude(dist) <= moveMag) {
			intersected = true;
			head.position = intersect;
		}
	}

	if(intersected) {
		tail.position += movement;
		if(magnitude(tail.position - origin) >= magnitude(head.position - origin)) {
			tail.position = head.position;
			active = false;
		}
	} else {
		head.position += movement;

		float curTracerLength = magnitude(head.position - tail.position);

		if(curTracerLength > tracerLength) {
			tail.position = head.position - ((head.position - tail.position) / curTracerLength * tracerLength);
		}
	}
}

void Bullet::draw(sf::RenderTarget &target, sf::RenderStates states) const {
	target.draw(tracer, states);
}

bool Bullet::IsColliding(const Entity &entity) const {
	return entity.IsColliding(tracer[0].position, tracer[1].position);
}

void Bullet::Damage(Unit &unit) {
	if(!hasDamaged) {
		unit.RemoveHealth(power);
		hasDamaged = false;
	}
}
