
#include "Game.h"
#include "Level.h"
#include "Player.h"
#include "MenuManager.h"

const sf::Time Game::fadeoutTime = sf::seconds(3.0f);
const sf::Time Game::introTime = sf::seconds(3.0f);
const std::string Game::firstLevel = "map.tmx";
const std::string Game::firstSpawn = "Init";
const std::string Game::windowTitle = "Flaming Wallhack";

Game::Game()
	: winWidth(800), winHeight(600)
	, window(sf::VideoMode::getDesktopMode(), Game::windowTitle)
	, menus(window.getSize())
	, view(sf::FloatRect(0.0f, 0.0f, (float)winWidth, (float)winHeight))
	, fadeoutRect(sf::Vector2f((float)winWidth, (float)winHeight))
{
	//Flag set to false when the game ends
	isRunning = false;
	paused = true;

	window.setView(view);

	fadeoutRect.setFillColor(sf::Color(0,0,0,0));
	fadeoutRect.setPosition(0,0);
	fadeoutTimer = sf::Time::Zero;

	cheatCounter = 0;
}

int Game::Run()
{
	isRunning = true;
	Init();
#ifdef HAVE_CONSOLE
	Intro();
#endif
	Loop();

	return 0;
}

void Game::Intro()
{
	/** Logo for the intro. */
	sf::Sprite introLogo;
	/** Texture for the logo. */
	sf::Texture introLogoTexture;
	/** Timer for the intro. */
	sf::Clock introTimer;

	introLogoTexture.loadFromFile("assets/img/logo.png");
	introLogo.setTexture(introLogoTexture);
	window.draw(introLogo);
	window.display();
	while(introTimer.getElapsedTime() <= introTime) { }
	window.clear();
}

void Game::Init()
{
	// --- INITIALIZATION ---
	//Turn off repeating keypresses (because there's a delay between pressing the key and
	// perceiving it as held down)
	window.setKeyRepeatEnabled(false);

	//Connect the exit button of the menus to the exit function here
	menus.connectExitFunc(&Game::Exit, this);
}

void Game::Loop()
{
	//Timer for regulating framerate - for an explanation of this, go to the end of
	// the main game loop
	sf::Clock frameTimer;
	//How many frames per second we want the game to run
	const int ticksPerSec = 60;
	//The time needed for that frame
	sf::Time frameTime = sf::microseconds(sf::seconds(1).asMicroseconds()/ticksPerSec);

	updateTimer.restart();

	while(isRunning)
	{
		//Reset the framerate regulation timer
		frameTimer.restart();

		Event();
		Update();
		Render();

		sf::Time timeSleep = frameTime - frameTimer.getElapsedTime();
		if(timeSleep.asMicroseconds() >= 0)
			sf::sleep(timeSleep);
		//printf("Frame length: %lld milliseconds\n", frameTimer.getElapsedTime());
		//If the time to sleep is less than 0, we don't have to do anything, though the game will
		// start to lag
	}
}

void Game::Event()
{
	// --- EVENT PROCESSING ---
	// Why is "event" a keyword in Visual Studio? We may never know...
	sf::Event anEvent;
	while(window.pollEvent(anEvent)) {
		//This event needs to go before the menu receives the input, so that 
		// the window can still close if the menu's open
		if(anEvent.type == sf::Event::Closed) {
			Exit();
		}
		//Pass the menu events and don't pass the game events if the menu is visible
		else if(menus.IsVisible()) {
			menus.HandleEvent(anEvent);
			//If the menu just turned invisible after handling an event, restart the
			// update timer so that entities don't think they were simulating while
			// the menu was up
			updateTimer.restart();
			if(!menus.IsVisible()) {
				paused = false;
			}
		}
		
		//If no level was loaded before and we are now in the game, load a level
		if(!paused && !levelManager.HasLoadedLevel()) {
			ToggleLoading();
			levelManager.LoadMap(firstLevel, firstSpawn);
			ToggleLoading();
		}
		switch(anEvent.type) {
		case sf::Event::Resized :
			menus.Resize(sf::Vector2f(anEvent.size.width, anEvent.size.height));
			//Everything else is taken care of elsewhere
			break;
		case sf::Event::MouseButtonPressed :
			if(!paused) {
				switch(anEvent.mouseButton.button) {
				case sf::Mouse::Left :
					levelManager.GetPlayer().Shoot(true);
					break;
				default:
					break;
				}
			}
			break;
		case sf::Event::MouseButtonReleased :
			if(!paused) {
				switch(anEvent.mouseButton.button) {
				case sf::Mouse::Left :
					levelManager.GetPlayer().Shoot(false);
					break;
				default:
					break;
				}
			}
			break;
		case sf::Event::MouseMoved :
			if(!paused) {
				sf::Vector2i rawMousePos(anEvent.mouseMove.x, anEvent.mouseMove.y);
				sf::Vector2i mousePos(window.mapPixelToCoords(rawMousePos));
				sf::Vector2f fMousePos(mousePos.x, mousePos.y);
				sf::Vector2f diff = fMousePos - levelManager.GetPlayer().GetPos();
				float angle = atan2f(diff.y, diff.x);
				levelManager.GetPlayer().SetFacing(angle);
			}
		case sf::Event::KeyPressed :
			//If the game is not paused, allow it to receive events
			if(!paused) {
				switch(anEvent.key.code) {
				case sf::Keyboard::W :
					levelManager.GetPlayer().MoveUp(true);
					break;
				case sf::Keyboard::S :
					levelManager.GetPlayer().MoveDown(true);
					break;
				case sf::Keyboard::A :
					levelManager.GetPlayer().MoveLeft(true);
					AddCheatCounter(9, -1);
					break;
				case sf::Keyboard::D :
					levelManager.GetPlayer().MoveRight(true);
					break;
				case sf::Keyboard::E :
					levelManager.GetCurrentLevel().DoActivate();
					break;
				case sf::Keyboard::Space :
					levelManager.GetPlayer().StartAttack();
					break;
				case sf::Keyboard::LShift :
				case sf::Keyboard::RShift :
					levelManager.GetCurrentLevel().GetPlayer().Sprint(true);
					break;
				case sf::Keyboard::Escape :
					menus.SetVisible(true);
					paused = true;
					break;
				case sf::Keyboard::Up :
					AddCheatCounter(0, 1);
					break;
				case sf::Keyboard::Down :
					AddCheatCounter(2, 3);
					break;
				case sf::Keyboard::Left :
					AddCheatCounter(4, 6);
					break;
				case sf::Keyboard::Right :
					AddCheatCounter(5, 7);
					break;
				case sf::Keyboard::B :
					AddCheatCounter(8, -1);
				default:
					break;
				}
			}
			break;
		case sf::Event::KeyReleased :
			//If the game is not paused, allow it to receive events
			if(!paused) {
				switch(anEvent.key.code) {
				case sf::Keyboard::W :
					levelManager.GetPlayer().MoveUp(false);
					break;
				case sf::Keyboard::S :
					levelManager.GetPlayer().MoveDown(false);
					break;
				case sf::Keyboard::A :
					levelManager.GetPlayer().MoveLeft(false);
					break;
				case sf::Keyboard::D :
					levelManager.GetPlayer().MoveRight(false);
					break;
				case sf::Keyboard::LShift :
				case sf::Keyboard::RShift :
					levelManager.GetPlayer().Sprint(false);
					break;
				default:
					break;
				}
				break;
			}
		default :
			break;
		}
	}
}

void Game::Update()
{
	//We have to store the update time and re-get it BEFORE passing it to the level
	// because the update function is the most time consuming; the time taken within
	// the function itself is not negligible, so we can't put the restart() after it
	//Also, we need to put this outside the "if(!paused)" block so that after the game
	// isn't paused, entities don't think that time passed within the pause
	//printf("Update length: %lld milliseconds\n", updateTimer.getElapsedTime());
	sf::Time updateTime = updateTimer.getElapsedTime();
	updateTimer.restart();

	if(cheatCounter == 10) {
		levelManager.GetPlayer().AddHealth(100);
		levelManager.GetPlayer().AddEnergy(100);
	}

	//Check win/lose conditions
	if(levelManager.HasLoadedLevel()) {
		bool win = false;
		bool lose = !levelManager.GetCurrentLevel().GetPlayer().IsAlive();

		if(win || lose) {
			fadeoutTimer += updateTime;
			if(win) {
				fadeoutRect.setFillColor(sf::Color(255,255,255,
					(sf::Uint8)(255*fadeoutTimer.asMicroseconds()/(float)fadeoutTime.asMicroseconds())));
			} else {
				fadeoutRect.setFillColor(sf::Color(0,0,0,
					(sf::Uint8)(255*fadeoutTimer.asMicroseconds()/(float)fadeoutTime.asMicroseconds())));
			}
			if(fadeoutTimer >= fadeoutTime) {
				if(win) {
					//Win sounds
				} else {
					//Lose sounds
				}
				fadeoutTimer = sf::Time::Zero;
				levelManager.Reset();
				menus.WinLose(win);
				menus.SetVisible(true);
			}
			paused = true;
		}
	} else {
		fadeoutRect.setFillColor(sf::Color(0,0,0,0));
	}

	if(!paused) {
		sf::Vector2f playerPos = levelManager.GetCurrentLevel().GetPlayer().GetPos();
		sf::Vector2i levelSize = levelManager.GetCurrentLevel().GetSize();
		sf::Vector2f viewSize = view.getSize();
		sf::Vector2f viewPos = view.getCenter();

		//If the level's about to transition, put up a loading screen
		if(levelManager.IsTransitioning()) {
			ToggleLoading();
			levelManager.Update(updateTime);
			ToggleLoading();
		} else {
			//Update the level
			levelManager.Update(updateTime);
		}

		//printf("player: (%g, %g) view: (%g, %g)\n", playerPos.x, playerPos.y, viewPos.x, viewPos.y);

		//Update the view to follow the player, but don't make it go offscreen
		if(playerPos.x - viewSize.x/2 >= 0 && playerPos.x + viewSize.x/2 <= levelSize.x)
			view.setCenter(sf::Vector2f(playerPos.x, viewPos.y));
		else if(playerPos.x - viewSize.x/2 < 0)
			view.setCenter(sf::Vector2f(viewSize.x/2, viewPos.y));
		else if(playerPos.x + viewSize.x/2 > levelSize.x)
			view.setCenter(sf::Vector2f(levelSize.x - viewSize.x/2, viewPos.y));

		viewPos = view.getCenter();

		if(playerPos.y - viewSize.y/2 >= 0 && playerPos.y + viewSize.y/2 <= levelSize.y)
			view.setCenter(sf::Vector2f(viewPos.x, playerPos.y));
		else if(playerPos.y - viewSize.y/2 < 0)
			view.setCenter(sf::Vector2f(viewPos.x, viewSize.y/2));
		else if(playerPos.y + viewSize.y/2 > levelSize.x)
			view.setCenter(sf::Vector2f(viewPos.x, levelSize.y - viewSize.y/2));

		window.setView(view);
	}
}

void Game::Render()
{
	// --- RENDERING ---
	//Clear the screen of what was drawn before
	// This is inefficient, but if we don't call it, then there will be artefacts from the
	// previous frame
	window.clear();

	if(levelManager.HasLoadedLevel()) {
		window.draw(levelManager.GetCurrentLevel());
	}

	//Render non-scrolling objects in the default view
	window.setView(window.getDefaultView());
	window.draw(fadeoutRect);
	window.setView(view);

	menus.Display(window);
	//Update the window after the drawing
	window.display();
}

void Game::Exit()
{
	isRunning = false;
}

void Game::ToggleLoading()
{
	menus.ToggleLoading();
	menus.SetVisible(!menus.IsVisible());
	//Render should really not have to be called here, but there's no good
	// way around it except for deferring loading yet another frame or
	// going to multithreaded (which is a good way, but too late for that)
	Render();
}

void Game::AddCheatCounter(int a, int b)
{
	if((cheatCounter == a || cheatCounter == b) && !(cheatCounter == 10 || cheatCounter == 11)) {
		cheatCounter++;
	} else if(!(cheatCounter == 10 || cheatCounter == 11)) {
		cheatCounter = 0;
	}
	printf("%d\n", cheatCounter);
}
