
#include "Gun.h"

#include "Utility/Utility.h"
#include "Level.h"

Gun::Gun() {
	/** Default values roughly correspond to a pistol */
	bulletLifetime = sf::seconds(5.0f);
	lifetimeVariance = sf::Time::Zero;
	gunCooldown = sf::seconds(0.02f);

	barrelDistance = 10.0f;
	power = 10.0f;
	speed = 1000.0f;
	powerVariance = 0.0f;
	angleVariance = 1.0f;
	speedVariance = 100.0f;
	tracerLengthOverride = -1;
	tracerLengthVariance = 0;

	automatic = false;
}

void Gun::setBulletLifetime(sf::Time lifetime) {
	this->bulletLifetime = lifetime;
}

void Gun::setBulletLIfetimeVariance(sf::Time variance) {
	this->lifetimeVariance = variance;
}

void Gun::setCooldown(sf::Time cooldown) {
	this->gunCooldown = cooldown;
}

void Gun::setPower(float power) {
	this->power = power;
}

void Gun::setPowerVariance(float powerVariance) {
	this->powerVariance = powerVariance;
}

void Gun::setAngleVariance(float angleVariance) {
	this->angleVariance = angleVariance;
}

void Gun::setSpeed(float speed) {
	this->speed = speed;
}

void Gun::setSpeedVariance(float speedVariance) {
	this->speedVariance = speedVariance;
}

void Gun::overrideTracerLenVariance(float tracerLenVariance) {
	this->tracerLengthVariance = tracerLenVariance;
}

void Gun::overrideTracerLen(float tracerLen) {
	this->tracerLengthOverride = tracerLen;
}

void Gun::ToggleFiring(bool fire) {
	firing = fire;
	if(!firing) {
		fired = false;
	}
}

void Gun::Update(Level &level, sf::Time delta, sf::Vector2f pos, float facing) {
	timer += delta;
	if(firing && timer >= gunCooldown && (automatic || (!automatic && !fired))) {
		float angle = facing + getRandom(-angleVariance/2.0f, angleVariance/2.0f);
		sf::Vector2f bulletPos(pos +
				sf::Vector2f(barrelDistance * cos(angle*TO_RAD), barrelDistance * sin(angle*TO_RAD)));
		sf::Time lifetime = bulletLifetime +
			sf::microseconds(getRandom(-lifetimeVariance.asMicroseconds(), lifetimeVariance.asMicroseconds()));
		float power = this->power + getRandom(-powerVariance/2.0f, powerVariance/2.0f);
		float speed = this->speed + getRandom(-speedVariance/2.0f, speedVariance/2.0f);
		float tracerLength = (tracerLengthOverride > 0 ? tracerLengthOverride :
				std::min(power * 3.0f, 200.0f)) + getRandom(-tracerLengthOverride/2.0f, tracerLengthOverride/2.0f);

		level.AddBullet(Bullet(bulletPos, lifetime, angle, speed, tracerLength, power));
		fired = true;
		timer = sf::Time::Zero;
	}
}
