
#include "Movable.h"

#include "Utility/Utility.h"
#include "Level.h"

Movable::Movable(std::string imgLoc, sf::IntRect collisionBox, sf::IntRect animBox)
	: Entity(imgLoc, collisionBox, animBox, true)
{}

void Movable::move(Level &level, sf::Vector2f movement) {
	ResolveXMove(level, movement);
	ResolveYMove(level, movement);
}

void Movable::ResolveXMove(Level &level, sf::Vector2f movement) {
	//Get the sides of the player's collision rectangle
	sf::Vector2f pos = sprite.getPosition();
	float top = pos.y - collisionBox.height/2;
	float left = pos.x - collisionBox.width/2;
	float bot = pos.y + collisionBox.height/2;
	float right = pos.x + collisionBox.width/2;
	sf::Vector2f topLeft(left, top);
	sf::Vector2f botRight(right, bot);
	sf::Vector2f topRight(right, top);
	sf::Vector2f botLeft(left, bot);
	//printf("(%g, %g), (%g, %g)\n", topLeft.x, topLeft.y, botRight.x, botRight.y);
	sf::Vector2f smallestMove(movement.x, 0.0f);

	for(int i = 0; i < 4; i++) {
		bool found = false;
		sf::Vector2f corner;
		sf::Vector2f collidePos(FLT_MAX, FLT_MAX);
		if(i == 0) {
			if(movement.x >= 0 && movement.y >= 0) {
				continue;
			}
			corner = topLeft;
		} else if(i == 1) {
			if(movement.x <= 0 && movement.y <= 0) {
				continue;
			}
			corner = botRight;
		} else if(i == 2) {
			if(movement.x <= 0 && movement.y >= 0) {
				continue;
			}
			corner = topRight;
		} else if(i == 3) {
			if(movement.x >= 0 && movement.y <= 0) {
				continue;
			}
			corner = botLeft;
		}

		found = level.GetCollideDir(corner, sf::Vector2f(sign(movement.x), 0.0f), collidePos);
		float xMove = collidePos.x - corner.x;
		xMove = sign(xMove) * std::max(0.0f, std::abs(xMove) - EPSILON*2.0f);
		if(found && std::abs(xMove) < std::abs(smallestMove.x)) {
			smallestMove.x = xMove;
		}
	}

	sprite.move(smallestMove);
}

void Movable::ResolveYMove(Level &level, sf::Vector2f movement) {
	//Get the sides of the player's collision rectangle
	sf::Vector2f pos = sprite.getPosition();
	float top = pos.y - collisionBox.height/2;
	float left = pos.x - collisionBox.width/2;
	float bot = pos.y + collisionBox.height/2;
	float right = pos.x + collisionBox.width/2;
	sf::Vector2f topLeft(left, top);
	sf::Vector2f botRight(right, bot);
	sf::Vector2f topRight(right, top);
	sf::Vector2f botLeft(left, bot);
	//printf("(%g, %g), (%g, %g)\n", topLeft.x, topLeft.y, botRight.x, botRight.y);
	sf::Vector2f smallestMove(0.0f, movement.y);

	for(int i = 0; i < 4; i++) {
		bool found = false;
		sf::Vector2f corner;
		sf::Vector2f collidePos(FLT_MAX, FLT_MAX);
		if(i == 0) {
			if(movement.x >= 0 && movement.y >= 0) {
				continue;
			}
			corner = topLeft;
		} else if(i == 1) {
			if(movement.x <= 0 && movement.y <= 0) {
				continue;
			}
			corner = botRight;
		} else if(i == 2) {
			if(movement.x <= 0 && movement.y >= 0) {
				continue;
			}
			corner = topRight;
		} else if(i == 3) {
			if(movement.x >= 0 && movement.y <= 0) {
				continue;
			}
			corner = botLeft;
		}

		found = level.GetCollideDir(corner, sf::Vector2f(0.0f, sign(movement.y)), collidePos);
		float yMove = collidePos.y - corner.y;
		yMove = sign(yMove) * std::max(0.0f, std::abs(yMove) - EPSILON*2.0f);
		if(found && std::abs(yMove) < std::abs(smallestMove.y)) {
			smallestMove.y = yMove;
		}
	}

	sprite.move(smallestMove);
}
