
#include "SoundManager.h"

std::unordered_map<std::string, sf::SoundBuffer> SoundManager::soundBuffers;

SoundManager::SoundManager()
{
	nextSound = 0;
}

bool SoundManager::AddSound(const std::string& soundLoc, const std::string& soundName, bool loop)
{
	//Ensure the sound is not already registered
	if(soundMap.find(soundName) == soundMap.end()) {
		//If the sound buffer has not already been registered, do so now
		auto soundBufferIter = soundBuffers.find(soundLoc);
		if(soundBufferIter == soundBuffers.end()) {
			sf::SoundBuffer buffer;
			if(buffer.loadFromFile(soundLoc)) {
				soundBufferIter->second = buffer;

				//Add the sound to the library of sounds
				soundMap.insert(std::make_pair<std::string, SoundRec>(
							std::string(soundName), SoundRec(soundBufferIter->second, loop)));
				return true;
			}
		}
	}
	//If the sound was already registered or we couldn't load the file, return false
	return false;
}

bool SoundManager::PlaySound(const std::string& soundName)
{
	auto soundIter = soundMap.find(soundName);
	if(soundIter != soundMap.end()) {
		soundIter->second.currentSound = nextSound;
		sf::Sound &sound = sounds[nextSound];
		/* Stop sound just in case */
		//sound.stop();
		sound.setLoop(soundIter->second.loop);
		sound.setBuffer(soundIter->second.buffer);
		sound.play();

		nextSound = (nextSound+1)%MAX_SOUNDS;
		return true;
	}
	return false;
}

bool SoundManager::StopSound(const std::string& soundName)
{
	auto soundIter = soundMap.find(soundName);
	if(soundIter != soundMap.end()) {
		int currentSound = soundIter->second.currentSound;
		if(currentSound >= 0) {
			//sounds[currentSound].stop();
		} else {
			return false;
		}
	}
	return true;
}

void SoundManager::SetSoundVolume(const std::string& soundName, float volume)
{
	auto soundIter = soundMap.find(soundName);
	if(soundIter != soundMap.end()) {
		int currentSound = soundIter->second.currentSound;
		if(currentSound >= 0) {
			sounds[currentSound].setVolume(volume);
		}
	}
}

void SoundManager::StopLastSound()
{
	sounds[nextSound-1].stop();
}
