
/*
 * \brief Manages sounds for entities.
 * \author Ed Lu
 * \date July 27 2014
 */

#ifndef __SOUNDSERVICE_H
#define __SOUNDSERVICE_H

#include <string>

class SoundService
{
public:
	/**
	 * Adds a sound to the library.
	 * \param soundLoc The location, on disk, of the sound.
	 * \param soundName The name to associate with the sound.
	 * \param loop Flag indicating whether the sound should loop or not; true for yes.
	 * \return True of the sound was successfully added, false otherwise.
	 */
	virtual bool AddSound(const std::string& soundLoc, const std::string& soundName, bool loop) = 0;

	/**
	 * Plays a sound.
	 * \param soundName The name of the sound to play.
	 * \return True if the sound exists, false otherwise.
	 */
	virtual bool PlaySound(const std::string& soundName) = 0;

	/**
	 * Stops playing a sound.
	 * \param soundName The name of the sound to play.
	 * \return True if the sound exists, false otherwise.
	 */
	virtual bool StopSound(const std::string& soundName) = 0;

	/**
	 * Sets a sound's volume.
	 * \param soundName The name of the sound.
	 * \param volume Volume of the sound, out of 100.
	 */
	virtual void SetSoundVolume(const std::string& soundName, float volume) = 0;

	/**
	 * Stops playing the last sound played.
	 */
	virtual void StopLastSound() = 0;
private:
};

#endif
