
#ifndef __NAVMESH_H
#define __NAVMESH_H

#include <SFML/System.hpp>

#include <vector>
#include <unordered_map>
#include <list>
#include <queue>

#include "Utility/Vec2fHash.h"
#include "Utility/Edge.h"

class TPPLPoly;

class Navmesh {
public:

	Navmesh();
	void LoadEdges(const std::vector<Edge<int>> &edges);

	std::vector<sf::Vector2f> findPath(sf::Vector2f start, sf::Vector2f end, float radius);

protected:

	struct AdjNode {
		AdjNode() {}
		AdjNode(unsigned poly, unsigned portal, unsigned adj_portal) :
			poly(poly), portal(portal), adj_portal(adj_portal) { }
		unsigned poly;
		//Portal on this shape, used to get to the _previous_ polygon
		unsigned portal;
		//Portal number on the previous shape to get to _this_ polygon
		unsigned adj_portal;
	};

	struct PathNode {
		PathNode() {}
		PathNode(unsigned poly, unsigned prev_portal, unsigned next_portal) :
			poly(poly), prev_portal(prev_portal), next_portal(next_portal) { }
		unsigned poly;
		//Portal on this shape, used to get to the previous polygon
		unsigned prev_portal;
		//Portal on this shape, used to get to the next polygon
		unsigned next_portal;
	};

	//first is g-val, second is h-val
	typedef std::pair<float, float> CostPair;

	float h_val(sf::Vector2f goal, AdjNode node);
	float g_val(sf::Vector2f start, sf::Vector2f end,
		CostPair p_cost, float h_val, AdjNode parent, AdjNode node,
		bool trust_parent);
    
	std::vector<PathNode> findPolyPath(sf::Vector2f from, sf::Vector2f to, float radius);
	std::vector<sf::Vector2f> getPath(sf::Vector2f start, sf::Vector2f end,
            std::vector<PathNode> polypath, float radius);
    std::vector<std::vector<AdjNode>> makeGraph(
            const std::vector<std::vector<sf::Vector2f>> &polys,
            const std::vector<Edge<int>> &edges);

	void mapEdges(const std::list<TPPLPoly> &polys);

	sf::Vector2f pushAway(sf::Vector2f point, float distance);
	sf::Vector2f getNextLeft(PathNode node, float radius);
	sf::Vector2f getNextRight(PathNode node, float radius);

	std::vector<Edge<int>> edges;
	std::vector<std::vector<sf::Vector2f> > polys;
	std::vector<std::vector<AdjNode> > adjacencyList;

	std::unordered_map<unsigned, std::pair<float, float>> prios;
	std::unordered_map<sf::Vector2f, std::pair<sf::Vector2f, sf::Vector2f>> edgeFinderMap;

	class CompareFirst {
	public:
		bool operator()(std::pair<float, AdjNode> p1, std::pair<float, AdjNode> p2) {
			return p1.first > p2.first;
		}
	};

	typedef std::priority_queue<std::pair<float, AdjNode>,
			std::vector<std::pair<float, AdjNode> >,
			CompareFirst> prioq_t;
};

#endif
