
#include "gtest/gtest.h"
#include "Utility/Utility.h"

TEST(test_onLine, basic_true) {
	ASSERT_TRUE(onLine(sf::Vector2f(10.0f, 10.0f),
				sf::Vector2f(10.0f, 20.0f),
				sf::Vector2f(10.0f, 15.0f)));
}

TEST(test_onLine, basic_false) {
	ASSERT_FALSE(onLine(sf::Vector2f(10.0f, 10.0f),
				sf::Vector2f(10.0f, 20.0f),
				sf::Vector2f(11.0f, 15.0f)));
	ASSERT_FALSE(onLine(sf::Vector2f(10.0f, 10.0f),
				sf::Vector2f(10.0f, 20.0f),
				sf::Vector2f(10.0f, 20.1f)));
}

TEST(test_onLine, on_endpoint) {
	ASSERT_TRUE(onLine(sf::Vector2f(10.0f, 10.0f),
				sf::Vector2f(10.0f, 20.0f),
				sf::Vector2f(10.0f, 20.0f)));
}

TEST(test_onLine, close) {
	ASSERT_TRUE(onLine(sf::Vector2f(10.0f, 10.0f),
				sf::Vector2f(18.0f, 14.0f),
				sf::Vector2f(14.0001f, 11.9999f)));
}

TEST(test_angleBetween, basic) {
	ASSERT_NEAR(angleBetween(sf::Vector2f(1.0f, 0.0f), sf::Vector2f(1.0f, 1.0f)),
			PI/4.0f, EPSILON);
	ASSERT_NEAR(angleBetween(sf::Vector2f(1.0f, 0.0f), sf::Vector2f(2.0f, 1.0f)),
			0.46365f, EPSILON);
	ASSERT_NEAR(angleBetween(sf::Vector2f(1.0f, 0.0f), sf::Vector2f(2.0f, -1.0f)),
			-0.46365f, EPSILON);
	ASSERT_NEAR(angleBetween(sf::Vector2f(-2.0f, -1.0f), sf::Vector2f(0.0f, -1.0f)),
			PI/2.0f - 0.46365f, EPSILON);
}

TEST(test_angleBetween, quadrants) {
	ASSERT_NEAR(angleBetween(sf::Vector2f(1.0f, -1.0f), sf::Vector2f(1.0f, 1.0f)),
			PI/2.0f, EPSILON);
	ASSERT_NEAR(angleBetween(sf::Vector2f(1.0f, -1.0f), sf::Vector2f(-1.0f, 1.0f)),
			PI, EPSILON);
	ASSERT_NEAR(angleBetween(sf::Vector2f(1.0f, -1.0f), sf::Vector2f(-1.0f, -1.0f)),
			-PI/2.0f, EPSILON);
	ASSERT_NEAR(angleBetween(sf::Vector2f(-1.0f, -1.0f), sf::Vector2f(-1.0f, -1.0f)),
			0.0f, EPSILON);
}

TEST(test_lineDistance_point, basic) {
	ASSERT_NEAR(lineDistance(sf::Vector2f(0.0f, 2.0f),
				sf::Vector2f(0.0f, 0.0f),
				sf::Vector2f(5.0f, 5.0f)),
			sqrt(2.0f), EPSILON);
	ASSERT_NEAR(lineDistance(sf::Vector2f(2.0f, 2.0f),
				sf::Vector2f(0.0f, 0.0f),
				sf::Vector2f(5.0f, 5.0f)),
			0.0f, EPSILON);
	ASSERT_NEAR(lineDistance(sf::Vector2f(3.0f, 3.0f),
				sf::Vector2f(2.0f, 2.0f),
				sf::Vector2f(6.0f, 10.0f)),
			1.0f/sqrt(5.0f), EPSILON);
}

TEST(test_lineDistance_point, endpoints) {
	ASSERT_NEAR(lineDistance(sf::Vector2f(-1.0f, 0.0f),
				sf::Vector2f(1.0f, 1.0f),
				sf::Vector2f(5.0f, 5.0f)),
			sqrt(5.0f), EPSILON);
	ASSERT_NEAR(lineDistance(sf::Vector2f(6.0f, 7.0f),
				sf::Vector2f(1.0f, 1.0f),
				sf::Vector2f(5.0f, 5.0f)),
			sqrt(5.0f), EPSILON);
}

TEST(test_lineDistance_line, basic) {
	ASSERT_NEAR(lineDistance(sf::Vector2f(-2.0f, 1.0f),
				sf::Vector2f(0.0f, 2.0f),
				sf::Vector2f(0.0f, 0.0f),
				sf::Vector2f(5.0f, 5.0f)),
			sqrt(2.0f), EPSILON);
	ASSERT_NEAR(lineDistance(sf::Vector2f (0.0f, 1.0f),
				sf::Vector2f(2.0f, 2.0f),
				sf::Vector2f(0.0f, 0.0f),
				sf::Vector2f(5.0f, 5.0f)),
			0.0f, EPSILON);
	ASSERT_NEAR(lineDistance(sf::Vector2f(0.0f, 5.0f),
				sf::Vector2f(3.0f, 3.0f),
				sf::Vector2f(2.0f, 2.0f),
				sf::Vector2f(6.0f, 10.0f)),
			1.0f/sqrt(5.0f), EPSILON);
}

TEST(test_lineDistance_line, parallel) {
	ASSERT_NEAR(lineDistance(sf::Vector2f(0.0f, 1.0f),
				sf::Vector2f(10.0f, 21.0f),
				sf::Vector2f(1.0f, 1.0f),
				sf::Vector2f(11.0f, 21.0f)),
			2 * sqrt(5) / 5, EPSILON);
}
