import sys
import pygame
import json

padding = 1

class Edge:
    def __init__(self, c1, c2):
        self.c1 = c1
        self.c2 = c2

    def asTuple(self):
        return (self.c1,self.c2)

    def default(self, o):
        return {'c1':self.c1,'c2':self.c2}

class Vec2:
    def __init__(self, x, y):
        self.x = x
        self.y = y

    def asTuple(self):
        return (self.x,self.y)

    def asDict(self):
        return {'x':self.x,'y':self.y}

def encode_things(thing):
    if isinstance(thing, Edge) or isinstance(thing, Vec2):
        return thing.__dict__
    return thing

def main():
    if len(sys.argv) < 2:
        print('Need to provide tileset filename as arg')
        return 1;

    xsize = int(input('x size: '))
    ysize = int(input('y size: '))

    pygame.init()

    screen = pygame.display.set_mode((xsize + padding*2, ysize + padding*2))
    tileset = pygame.image.load(sys.argv[1])
    tilesetSize = tileset.get_size()
    tilesetSize = Vec2(int(tilesetSize[0]/xsize), int(tilesetSize[1]/ysize))
    tileSize = Vec2(xsize, ysize)

    print('loaded tileset ', sys.argv[1], 'with size (', tilesetSize.x, ', ', tilesetSize.y, ')')

    annotation = {}
    alldone = False

    for t_y in range(tilesetSize.y):
        if alldone:
            break
        for t_x in range(tilesetSize.x):
            if alldone:
                break

            print('annotating tile at {}'.format((t_x,t_y)))
            points = []
            edges = []
            special = None
            redraw(screen, tileset, points, edges, Vec2(t_x, t_y), tileSize)
            done = False

            while not done:
                action = input('action: ')
                if action == 'insert' or action == 'i':
                    x = input('X point: ')
                    y = input('Y point: ')
                    point = get_point(x, y, tileSize)
                    if point is None:
                        print('error: some format was not good')
                        continue

                    points.append(point)
                    print('inserted point {}'.format(point))

                    if len(points) > 1:
                        edge = get_edge(len(points)-2, len(points)-1)
                        edges.append(edge)
                        print('inserting edge {}'.format(edge))

                elif action == 'undo' or action == 'u':
                    if len(points) > 0:
                        points.pop();
                        edges = [x for x in edges if (x.c1 != len(points) and x.c2 != len(points))]
                    else:
                        print("error: nothing to undo")

                elif action == 'fulltile' or action == 'f':
                    special = 'Full'
                    done = True
                elif action == 'close' or action == 'c':
                    edges.append(get_edge(len(points)-1, 0))
                    done = True
                elif action == 'notile' or action == 'n':
                    done = True
                elif action == 'alldone':
                    done = True
                    alldone = True
                else:
                    print('error: action {} was not found'.format(action))

                redraw(screen, tileset, points, edges, Vec2(t_x, t_y), tileSize)

            obj = {}
            if len(points) is not 0:
                obj['points'] = points
            if len(edges) is not 0:
                obj['edges'] = edges
            if special is not None:
                obj['special'] = special

            if len(obj) > 0:
                annotation[t_y*tilesetSize.x + t_x] = obj

    print(json.dumps(annotation, default=encode_things))

def redraw(screen, tileset, points, edges, tile, size):
    screen.fill((0,0,0))
    screen.blit(tileset, (padding,padding), pygame.Rect((tile.x*size.x, tile.y*size.y), size.asTuple()))

    for edge in edges:
        p1 = (points[edge.c1].x + padding, points[edge.c1].y + padding)
        p2 = (points[edge.c2].x + padding, points[edge.c2].y + padding)
        pygame.draw.line(screen, 0xff8888, p1, p2)

    pygame.display.flip()


def get_point(x, y, tileSize):
    if x[0] == 'r':
        x = tileSize.x
    elif x[0] == 'l':
        x = 0
    else:
        try:
            x = int(x)
        except ValueError:
            return None

    if y[0] == 'b':
        y = tileSize.y
    elif y[0] == 't':
        y = 0
    else:
        try:
            y = int(y)
        except ValueError:
            return None

    return Vec2(x, y)

def get_edge(c1, c2):
    return Edge(c1,c2)

if __name__ == '__main__':
    main()
